//
//
// jajajajan single app
//
//
angular.module('jajajajan', ['ionic', 'slick'])

// 作品ID
.constant('SKU', 'threepigs')

// JASRACナンバー
.constant('JASRAC', {
  'ios': '9014535001Y43032',
  'android': '9014535003Y43032',
  'bandai': ''
})

// シリーズ一覧JSON
.constant('SERIES_LIST_JSON', 'https://jajajajan-single-app.s3.amazonaws.com/series-v3-' + ionic.Platform.platform() + '.json')

// AdMob設定JSON
.constant('ADMOB_JSON', 'https://jajajajan-single-app.s3.amazonaws.com/admob-v2.json')

// Google Analytics
.constant('TRACKING_UA', 'UA-41455020-5')

// プラットフォーム(Tapmeのとき)
//.constant('PLATFORM', 'bandai')

// プラットフォーム(Tapme以外のとき)
.constant('PLATFORM', ionic.Platform.platform())

.config(function($stateProvider, $urlRouterProvider, $sceDelegateProvider) {
  $urlRouterProvider.otherwise('/init');
  $stateProvider
    .state('init', {
      url: '/init',
      controller: function() {}
    })
    .state('player', {
      url: '/player',
      controller: 'PlayerCtrl',
      templateUrl: 'player.html'
    })
    .state('coloring', {
      url: '/coloring/:page',
      controller: 'ColoringCtrl',
      templateUrl: 'coloring.html'
    })
    ;

  $sceDelegateProvider.resourceUrlWhitelist([
    'self',
    'http://localhost:*/**',
    'cdvfile://localhost/**',
    'file:///data/data/**',
    'https://s3-ap-northeast-1.amazonaws.com/jajajajan-single-app/**',
    'https://jajajajan-single-app.s3.amazonaws.com/**'
  ]);
})

.controller('PlayerCtrl', function($scope, $state, $timeout, $q, $ionicModal, $ionicLoading, $http, SKU, JASRAC, SERIES_LIST_JSON, ADMOB_JSON, PLATFORM) {
  var cover  = document.querySelector('#cover')
    , ja     = document.querySelector('#video-ja')
    , en     = document.querySelector('#video-en')
    , seeker = document.querySelector('#seeker-slider')
    , seekerDragging = false
    , seekerTime = new Date()
    , seekerTimeDisplay = document.querySelector('#seeker-time')
    , autoHidePromise = null
    , platform = PLATFORM
    ;

  $scope.platform = platform;
  $scope.jasracNumber = JASRAC[platform];

  /**
   * ユーザー設定を読込
   */
  function loadPreferences() {
    $scope.preferences = {
      repeats: 'off',
      launchCount: 1,
      interstitialCount: 0
    };

    if (!$scope.$root.db) { return; }

    $scope.$root.db.upsert('preferences', function(preferences) {
      if (!preferences.repeats) {
        preferences.repeats = 'off';
      }
      if (!preferences.interstitialCount) {
        preferences.interstitialCount = 0;
      }
      $scope.preferences = preferences;
      return preferences;
    });
  }

  /**
   * ユーザー設定を保存
   */
  function savePreferences() {
    if (!$scope.$root.db) { return; }

    $scope.$root.db.upsert('preferences', function(preferences) {
      preferences.repeats = $scope.preferences.repeats;
      preferences.interstitialCount = $scope.preferences.interstitialCount;
      return preferences;
    });
  }

  function videoEnded() {
    if ($scope.preferences && $scope.preferences.repeats == 'on') {
      $scope.currentVideo.currentTime = 0;
      $scope.currentVideo.play();
      if (window.analytics) {
        analytics.trackEvent('Video', 'repeated');
        analytics.trackView($scope.currentLayer + '-' + $scope.$root.lang);
      }
    } else {
      $scope.currentLayer = 'cover';
      $scope.$digest();
      if (window.analytics) {
        analytics.trackEvent('Video', 'not_repeated');
        analytics.trackView($scope.currentLayer);
      }
    }
  }

  function videoProgress() {
    if (!seekerDragging) {
      seeker.max = this.duration;
      seeker.value = this.currentTime;
      seekerTime.setTime(this.currentTime * 1000);
      seekerTimeDisplay.innerHTML = seekerTime.toString('mm:ss');
    }
  }

  function videoStarted() {
    autoHide();
  }

  function autoHide() {
    cancelAutoHide();
    autoHidePromise = $timeout(function() {
      $scope.hideVideoControllers = true;
    }, 3000);
  }

  function cancelAutoHide() {
    if (autoHidePromise) {
      $timeout.cancel(autoHidePromise);
    }
  }

  function videoTouched() {
    cancelAutoHide();

    $timeout(function() {
      $scope.hideVideoControllers = !$scope.hideVideoControllers;
    });
  }

  seeker.addEventListener('touchstart', function() {
    cancelAutoHide();
    seekerDragging = true;
  });

  seeker.addEventListener('touchend', function() {
    seekerDragging = false;
  });

  seeker.addEventListener('change', function() {
    $scope.currentVideo.currentTime = this.value;
    seekerDragging = false;
  });

  $scope.currentLayer = 'cover';
  if (window.analytics) {
    analytics.trackView($scope.currentLayer);
    $scope.$on('application.resume', function() {
      analytics.trackView($scope.currentLayer);
    });
  }

  $scope.appBarVisible = false;
  $scope.toggleAppBar = _.throttle(function() {
    $scope.appBarVisible = !$scope.appBarVisible;
  }, 600);

  $scope.listenOnEnd = function() {
    var splash = document.querySelector('#splash-video');
    if (splash) {
      splash.addEventListener('play', function() {
        if (navigator.splashscreen) {
          navigator.splashscreen.hide();
        }
      });
      splash.addEventListener('ended', function() {
        $scope.currentLayer = 'cover';
        if (window.analytics) { analytics.trackView($scope.currentLayer); }
        this.style.display = 'none';
        $scope.$digest();
      });
    }
  };

  ja.addEventListener('ended', videoEnded);
  ja.addEventListener('playing', videoStarted);
  ja.addEventListener('timeupdate', videoProgress);
  ja.addEventListener('touchend', _.throttle(videoTouched, 1000));

  en.addEventListener('ended', videoEnded);
  en.addEventListener('playing', videoStarted);
  en.addEventListener('timeupdate', videoProgress);
  en.addEventListener('touchend', _.throttle(videoTouched, 1000));

  function _play(lang) {
    var video = document.querySelector('#video-' + lang);
    $timeout(function() {
      video.play();
      $scope.hideVideoControllers = false;
      $scope.currentVideo = video;
      $scope.selectedLang = lang;
      $scope.toggleButtonState = 'pause';
      $scope.currentLayer = 'video';
      if (window.analytics) { analytics.trackView($scope.currentLayer + '-' + lang); }
    }, 500);
  }
  $scope.play = _.throttle(function(lang) {
    if (PLATFORM == 'android' && !$scope.$root.requestVideoFiles.resolved) {
      $ionicLoading.show({
        template: $scope.$root.lang == 'en' ? 'downloading' : 'ダウンロード中'
      });
      $q.all($scope.$root.requestVideoFiles()).then(function() {
        $scope.$root.requestVideoFiles.resolved = true;
        _play(lang);
      }).catch(function() {
        var msg = $scope.$root.lang == 'en' ? 'Failed to download. Please check your network connection.' : 'ダウンロードに失敗しました。ネットワーク接続をご確認ください。';
        alert(msg);
      })['finally'](function() {
        $ionicLoading.hide();
      });
    } else {
      _play(lang);
    }
  }, 500);

  $scope.togglePlay = function() {
    var video = $scope.currentVideo;

    if (video.paused) {
      video.play();
      $scope.toggleButtonState = 'pause';
    } else {
      video.pause();
      $scope.toggleButtonState = 'play';
    }
  };

  $scope.back = function() {
    $scope.currentVideo.pause();
    $scope.currentLayer = 'cover';
    if (window.analytics) { analytics.trackView($scope.currentLayer); }
  };

  $scope.rewind = function(amount) {
    $scope.currentVideo.pause();
    $scope.currentVideo.currentTime = Math.max(0, $scope.currentVideo.currentTime - amount);
    $scope.currentVideo.play();
  };

  $scope.periodicRewind = function() {
    cancelAutoHide();
    $scope.currentVideo.pause();
    $scope.currentVideo.currentTime = Math.max(0, $scope.currentVideo.currentTime - 3);
    $scope.periodicRewindPromise = $timeout($scope.periodicRewind, 200);
  };

  $scope.stopPeriodicRewind = function() {
    if ($scope.periodicRewindPromise) {
      $timeout.cancel($scope.periodicRewindPromise);
    }
    $scope.currentVideo.play();
  };

  $scope.forward = function(amount) {
    if ($scope.currentVideo.duration - $scope.currentVideo.currentTime < 4) { return; }
    if (!($scope.currentVideo && $scope.currentVideo.currentTime)) { return; }
    $scope.currentVideo.pause();
    $scope.currentVideo.currentTime = Math.min($scope.currentVideo.duration - 3, $scope.currentVideo.currentTime + amount);
    $scope.currentVideo.play();
  };

  $scope.periodicForward = function() {
    if ($scope.currentVideo.duration - $scope.currentVideo.currentTime < 4) { return; }
    cancelAutoHide();
    $scope.currentVideo.pause();
    $scope.currentVideo.currentTime = Math.min($scope.currentVideo.duration - 3, $scope.currentVideo.currentTime + 3);
    $scope.periodicForwardPromise = $timeout($scope.periodicForward, 200);
  };

  $scope.stopPeriodicForward = function() {
    if ($scope.periodicForwardPromise) {
      $timeout.cancel($scope.periodicForwardPromise);
    }
    $scope.currentVideo.play();
  };

  $scope.toggleRepeatState = function() {
    switch ($scope.preferences.repeats) {
      case 'off':
        $scope.preferences.repeats = 'on';
        break;
      case 'on':
        $scope.preferences.repeats = 'off';
        break;
      default:
        $scope.preferences.repeats = 'off';
        break;
    }
    savePreferences();
  };

  $scope.openMenu = function() {
    if ('cordova' in window && cordova.plugins && cordova.plugins.InterstitialAd && $scope.admobHomeBannerEnabled) {
        cordova.plugins.InterstitialAd.hideHomeBanner(function() {
            $timeout(function() {
              $scope.homeBannerIsVisible = false;
            });
        });
    }

    $scope.openParentGate().then(function() {
      $scope.menuIsOpen = true;
      $scope.menuWillOpen = true;
      if (window.analytics) { analytics.trackEvent('Menu', 'opened'); }
    }).catch(function() {
      if ('cordova' in window && cordova.plugins && cordova.plugins.InterstitialAd && $scope.admobHomeBannerEnabled) {
          cordova.plugins.InterstitialAd.showHomeBanner(function() {
            $timeout(function() {
              $scope.homeBannerIsVisible = true;
            });
          });
      }
    });
  };

  $scope.openInfo = function() {
    if ('cordova' in window && cordova.plugins && cordova.plugins.InterstitialAd && $scope.admobHomeBannerEnabled) {
        cordova.plugins.InterstitialAd.hideHomeBanner(function() {
            $timeout(function() {
              $scope.homeBannerIsVisible = false;
            });
        });
    }

    $scope.showInfo().then(function() {
      $scope.menuIsOpen = true;
      $scope.menuWillOpen = true;
      if (window.analytics) { analytics.trackEvent('Menu', 'opened'); }
    }).catch(function() {
      if ('cordova' in window && cordova.plugins && cordova.plugins.InterstitialAd && $scope.admobHomeBannerEnabled) {
          cordova.plugins.InterstitialAd.showHomeBanner(function() {
            $timeout(function() {
              $scope.homeBannerIsVisible = true;
            });
          });
      }
    });
  };

  $scope.closeMenu = function() {
    $scope.menuWillOpen = false;
    $timeout(function() {
      $scope.menuIsOpen = false;

      if ('cordova' in window && cordova.plugins && cordova.plugins.InterstitialAd && $scope.admobHomeBannerEnabled) {
          cordova.plugins.InterstitialAd.showHomeBanner(function() {
            $timeout(function() {
              $scope.homeBannerIsVisible = true;
            });
          });
      }
    }, 800);
  };

  /**
   * AdMobを閉じるボタン
   */
  $scope.hideHomeBanner = function() {
    cordova.plugins.InterstitialAd.closeHomeBanner(function() {
      $timeout(function() {
        $scope.homeBannerIsVisible = false;
      });
    });
  };
  $scope.hideVideoBanner = function() {
    cordova.plugins.InterstitialAd.closeVideoBanner(function() {
      $timeout(function() {
        $scope.videoBannerIsVisible = false;
      });
    });
  };

  /**
   * layerの遷移をwatch
   */
  function watchCurrentLayer(current, prev) {
    if (current == 'cover' && prev == 'video') {
      if ('cordova' in window && cordova.plugins && cordova.plugins.InterstitialAd && $scope.admobInterstitialEnabled) {
        if ($scope.preferences && ($scope.preferences.interstitialCount++ % 3) == 0) {
          cordova.plugins.InterstitialAd.showInterstitial();
        }
      }
      if ('cordova' in window && cordova.plugins && cordova.plugins.InterstitialAd) {
        cordova.plugins.InterstitialAd.hideVideoBanner(function() {
          $timeout(function() {
            $scope.videoBannerIsVisible = false;
          });
        });
        if ($scope.admobHomeBannerEnabled) {
          cordova.plugins.InterstitialAd.showHomeBanner(function() {
            $timeout(function() {
              $scope.homeBannerIsVisible = true;
            });
          });
        }
      }
      watchCurrentLayer.timeoutPromise = $timeout(function() {
        angular.element(document.querySelectorAll('video')).addClass('hidden');
        $scope.selectedLang = null;
      }, 600);
    }

    if (current == 'video') {
      if ('cordova' in window && cordova.plugins && cordova.plugins.InterstitialAd) {
        cordova.plugins.InterstitialAd.hideHomeBanner(function() {
          $timeout(function() {
            $scope.homeBannerIsVisible = false;
          });
        });
      }
      $timeout.cancel(watchCurrentLayer.timeoutPromise);
      angular.element(document.querySelectorAll('video')).removeClass('hidden');
    }
  }

  $scope.$watch('currentLayer', watchCurrentLayer);

  /**
   * ホームバナーはenableされたときにリクエスト
   */
  $scope.$watch('admobHomeBannerEnabled', function(current, prev) {
    if (current && 'cordova' in window && cordova.plugins && cordova.plugins.InterstitialAd && $scope.admobHomeBannerEnabled) {
        cordova.plugins.InterstitialAd.showHomeBanner(function() {
          $timeout(function() {
            $scope.homeBannerIsVisible = true;
          });
        });
    }

    if (!$scope.$root.isLaunch && $scope.hasInformation) {
      $scope.$root.isLaunch = true
      $timeout(function() {
        $scope.openInfo();
      }, 1000);
    };

  });

  $scope.$watch('hideVideoControllers', function(current, prev) {
    if ($scope.currentLayer == 'video') {
      if (current && !prev && 'cordova' in window && cordova.plugins && cordova.plugins.InterstitialAd && $scope.admobVideoBannerEnabled) {
          cordova.plugins.InterstitialAd.showVideoBanner(function() {
            $timeout(function() {
              $scope.videoBannerIsVisible = true;
            });
          });
      }
      if (!current && prev && 'cordova' in window && cordova.plugins && cordova.plugins.InterstitialAd && $scope.admobVideoBannerEnabled) {
          cordova.plugins.InterstitialAd.hideVideoBanner(function() {
            $timeout(function() {
              $scope.videoBannerIsVisible = false;
            });
          });
      }
    }
    
  });

  /**
   * ペアレントゲートを開く
   * @return {Promise}
   */
  $scope.openParentGate = function() {
    var deferred = $q.defer();
    $scope.$root.parentGateExist = true;
    $scope.$root.parentGateActive = true;
    $scope.$root.deferredParentGateTarget = deferred;
    return deferred.promise;
  };

  /**
   * ペアレントゲートを開く
   * @return {Promise}
   */
  $scope.$root.openParentGate = function() {
    $scope.$root.closeInfoGate();
    var deferred = $q.defer();
    $scope.$root.parentGateExist = true;
    $scope.$root.parentGateActive = true;
    $scope.$root.deferredParentGateTarget = deferred;

    if ('cordova' in window && cordova.plugins && cordova.plugins.InterstitialAd && $scope.admobHomeBannerEnabled) {
        cordova.plugins.InterstitialAd.hideHomeBanner(function() {
          $timeout(function() {
            $scope.homeBannerIsVisible = false;
          });
        });
    }

    return deferred.promise;
  };

  /**
   * ペアレントゲートを閉じる
   * @param {boolean} success 回答が成功したかどうか.
   */
  $scope.$root.closeParentGate = function(success) {
    $scope.$root.parentGateActive = false;
    $timeout(function() {
      $scope.$root.parentGateExist = false;
    }, 800);

    if ($scope.$root.deferredParentGateTarget) {
      var callback = success ? 'resolve' : 'reject';
      $scope.$root.deferredParentGateTarget[callback]();
    }
  };

  /**
   * お知らせを表示
   * @return {Promise}
   */
  $scope.showInfo = function() {
    var deferred = $q.defer();
    $scope.$root.infoGateExist = true;
    $scope.$root.infoGateActive = true;
    $scope.$root.deferredInfoGateTarget = deferred;
    return deferred.promise;
  };

  /**
   * お知らせを閉じる
   * @param {boolean} success 回答が成功したかどうか.
   */
  $scope.$root.closeInfoGate = function(success) {
    $scope.$root.infoGateActive = false;
    $timeout(function() {
      $scope.$root.infoGateExist = false;
    }, 800);

    if ($scope.$root.deferredInfoTarget) {
      var callback = success ? 'resolve' : 'reject';
      $scope.$root.deferredInfoGateTarget[callback]();
    }

    if ('cordova' in window && cordova.plugins && cordova.plugins.InterstitialAd && $scope.admobHomeBannerEnabled) {
        cordova.plugins.InterstitialAd.showHomeBanner(function() {
          $timeout(function() {
            $scope.homeBannerIsVisible = true;
          });
        });
    }
  };

  /**
   * シリーズ一覧Modal
   */
  $ionicModal.fromTemplateUrl('series.html', {
    scope: $scope
  }).then(function(modal) {
    $scope.jajajajanSeries = modal;
  });
  $scope.$on('$destroy', function() {
    $scope.jajajajanSeries.remove();
  });
  $scope.$on('modal.shown', function() {
    if (window.analytics) { analytics.trackEvent('SeriesList', 'opened'); }
    if ('cordova' in window && cordova.plugins && cordova.plugins.InterstitialAd && $scope.admobHomeBannerEnabled) {
        cordova.plugins.InterstitialAd.hideHomeBanner(function() {
          $timeout(function() {
            $scope.homeBannerIsVisible = false;
          });
        });
    }
  });
  $scope.$on('modal.hidden', function() {
    if ('cordova' in window && cordova.plugins && cordova.plugins.InterstitialAd && $scope.admobHomeBannerEnabled) {
        cordova.plugins.InterstitialAd.showHomeBanner(function() {
            $timeout(function() {
              $scope.homeBannerIsVisible = true;
            });
        });
    }
  });

  /**
   *
   */
  function hasStoreURLAndScheme(book) {
    var storeURL = book.storeURL
      , scheme = book.availability
      , notSelf = book.sku != SKU
      ;
    return storeURL && scheme && notSelf;
  }

  /**
   * シリーズ一覧、AdMob設定更新
   */
  function updateOnlineConfig() {
    $http.get(SERIES_LIST_JSON + '?t=' + Date.now())
      .success(function(data) {
        var lang = $scope.$root.lang;
        var titles = _(data[lang].titles).filter(hasStoreURLAndScheme).value();
        var pickup = _([data[lang].pickup]).filter(hasStoreURLAndScheme).first();
        var shuffle = data.shuffle;
        $scope.hasInformation = data.enabled_information;
        $scope.hasSeriesBooks = titles.length || (pickup && pickup.title);
        $scope.$root.seriesBooks = {
          titles: shuffle ? knuthShuffle(titles) : titles,
          pickup: pickup
        };
      });

    $http.get(ADMOB_JSON + '?t=' + Date.now())
      .success(function(data) {
        try {
          var flags = data[platform][SKU];


          if ($scope.preferences && $scope.preferences.launchCount >= 3) {
            $scope.admobInterstitialEnabled = flags.interstitial;
            $scope.admobHomeBannerEnabled = flags.home_banner;
            $scope.admobVideoBannerEnabled = flags.video_banner;
            if (flags.home_banner || flags.video_banner) {
              var adId  = data.bannerAdUnitID[platform];
              if (!adId) { return }
              cordova.plugins.InterstitialAd.createBanner(adId, $scope);
            }
          }

          // Google Conversion Tracking
          if (flags.conversion_id && flags.conversion_label && flags.conversion_value && (flags.conversion_repeatable === true || flags.conversion_repeatable === false)) {
            GappTrack.track(flags.conversion_id, flags.conversion_label, flags.conversion_value, flags.conversion_repeatable)
          }
        } catch (e) {
          console.log(e);
        }
      });
  }
  // バックグラウンドからの復帰時も更新
  $scope.$on('application.resume', function() {
    loadPreferences();
    updateOnlineConfig();
  });
  // 起動時にオンライン設定初期化
  loadPreferences();
  updateOnlineConfig();

  // pause時
  $scope.$on('application.pause', function() {
    savePreferences();
    console.log('application.pause');
  });

  /**
   * シリーズタップ
   */
  $scope.openBook = function(book) {
    if (window.analytics) { analytics.trackEvent('SeriesList', 'tapped', book.sku); }
    var scheme = book.availability;
    var store = book.storeURL;
    if ('startApp' in navigator && scheme) {
      navigator.startApp.check(scheme, function() {
        navigator.startApp.start(scheme);
      }, function() {
        $scope.openParentGate().then(function() {
          window.open(store, '_system');
        }).catch(function() {
        });
        $scope.$root.$digest();
      });
    } else {
      $scope.openParentGate().then(function() {
        window.open(store, '_system');
      }).catch(function() {
      });
    }
  };

  /**
   * Facebookページを開く
   */
  $scope.openFacebookPage = function() {
    var scheme = 'fb://'
    if ('startApp' in navigator && scheme) {
      navigator.startApp.check(scheme, function() {
        navigator.startApp.start('fb://profile/496087753797813');
      }, function() {
        window.open('https://www.facebook.com/496087753797813', '_system');
      });
    } else {
      window.open('https://www.facebook.com/496087753797813', '_system');
    }
    return false;
  };

  /**
   * ぬりえに移動
   */
  $scope.coloring = function() {
    $state.go('coloring', {page: 1});
  };

  /**
   * SplashMovie閉じる
   */
  $timeout(function() {
    'cordova' in window && cordova.exec(null, null, "SplashMovie", "hide", []);
  }, 500);
})

/**
 * ぬりえコントローラー
 */
.controller('ColoringCtrl', function($scope, $state, $stateParams, $timeout, PLATFORM, $ionicPopup) {
  $scope.$on('$ionicView.enter', function() {
    if ('cordova' in window && cordova.plugins && cordova.plugins.InterstitialAd) {
        AdMob.hideBanner();
    }

    //sounds.play('coloring_bgm', function(id) { $scope.bgm_id = id });
    sounds.coloring_bgm.play().loop();

    var penWidth = ionic.Platform.isTablet ? 16 : 8
    var ratio = Math.max(window.devicePixelRatio || 1, 1);
    var canvas = document.getElementById('coloring-canvas');
    var rainbow = document.getElementById('rainbow-canvas');
    var tmp = document.getElementById('tmp-canvas');
    tmp.globalCompositeOperation = 'source-atop';
    var particle = document.getElementById('particle-canvas');
    var gradient = chroma.scale(['red', 'orange', 'yellow', 'green', 'blue', 'indigo', 'violet', 'red']);
    canvas.width = particle.width = rainbow.width = tmp.width = canvas.offsetWidth * ratio;
    canvas.height = particle.height = rainbow.height = tmp.height = canvas.offsetHeight * ratio;
    canvas.getContext('2d').scale(ratio, ratio);
    particle.getContext('2d').globalCompositeOperation = 'lighter';
    rainbow.getContext('2d').scale(ratio, ratio);
    tmp.getContext('2d').scale(ratio, ratio);
    $scope.signaturePad = new SignaturePad(canvas, {
      minWidth: penWidth * .75,
      maxWidth: penWidth,
      penColor: $scope.currentColor
    });
    $scope.rainbowPad = new SignaturePad(rainbow, {
      minWidth: penWidth * .75,
      maxWidth: penWidth,
      penColor: 'black'
    });
    $scope.rainbowPad.gradientScale = 0;

    /**
     * Proton.jsのtouch対応
     */
    if (! ('_initEventHandler' in Proton.FollowEmitter.prototype)) {
      Proton.FollowEmitter.prototype._initEventHandler = Proton.FollowEmitter.prototype.initEventHandler;
    }
    Proton.FollowEmitter.prototype.initEventHandler = function() {
      var self = this;
      this._initEventHandler();
      this.mouseTarget.addEventListener('touchmove', _.throttle(function(event) {
        self.mousemove.call(self, {layerX: event.touches[0].clientX * ratio, layerY: event.touches[0].clientY * ratio});

        // 虹色ペンの場合
        if ($scope.currentColor == 'rainbow' && $scope.currentSlide != 3) {
          $scope.rainbowPad._handleTouchMove(event);
          tmp.width = tmp.width; // clear context
          var ctx = tmp.getContext('2d');
          ctx.drawImage(rainbow, 0, 0);
          ctx.globalCompositeOperation = 'source-atop';
          drawImageProp(ctx, $scope.currentColoredImage, 0, 0, canvas.width / ratio, canvas.height / ratio);
          canvas.getContext('2d').drawImage(tmp, 0, 0);
        } else {
          if ($scope.currentColor == 'rainbow' && $scope.currentSlide == 3) {
            // フリーページ(真っ白)のぬりえのとき
            $scope.signaturePad._ctx.fillStyle = gradient((($scope.rainbowPad.gradientScale += 1) % 100) / 100).hex();
          }
          // 指定色ペン
          $scope.signaturePad._handleTouchMove(event);
        }
      }, PLATFORM == 'android' ? 100 : 33), false);

      this.mouseTarget.addEventListener('touchstart', function(event) {
        $scope.currentSlide = $('#slick').slick('slickCurrentSlide');
        if ($scope.currentColor == 'rainbow' && $scope.currentSlide != 3) {
          $scope.rainbowPad._handleTouchStart(event);
          $scope.currentColoredImage = $scope.coloredImages[$scope.currentSlide];
          rainbow.width = rainbow.width; // clear context
          tmp.width = tmp.width; // clear context
        } else {
          $scope.signaturePad._handleTouchStart(event);
        }
        //sounds.play('drawing', function(id) { $scope.drawing_sound_id = id });
        sounds.drawing.play().loop();
      }, false);

      this.mouseTarget.addEventListener('touchend', function(event) {
        if ($scope.currentColor == 'rainbow' && $scope.currentSlide != 3) {
          $scope.rainbowPad._handleTouchEnd(event);
        } else {
          $scope.signaturePad._handleTouchEnd(event);
        }
        //sounds.stop($scope.drawing_sound_id);
        sounds.drawing.stop();
      }, false);
    };

    $scope.proton = new Proton();
    var emitter = new Proton.FollowEmitter(particle);
    emitter.rate = new Proton.Rate(Proton.getSpan(1, 5));
    emitter.addInitialize(new Proton.Mass(1));
    emitter.addInitialize(new Proton.Radius(3, 10));
    emitter.addInitialize(new Proton.Life(1, 3));
    emitter.addInitialize(new Proton.Velocity(new Proton.Span(-1, 1), new Proton.Span(-3, 0), 'vector'));
    emitter.addBehaviour(new Proton.Gravity(9.8));
    emitter.addBehaviour(new Proton.Color('random'));
    emitter.addBehaviour(new Proton.RandomDrift(30, 0, .035));
    emitter.addBehaviour(new Proton.Alpha(1, -0.5, Infinity, Proton.easeOutQuad));
    emitter.emit();
    $scope.proton.addEmitter(emitter);
    $scope.renderer = new Proton.Renderer('canvas', $scope.proton, particle);
    $scope.renderer.start();

    $scope.tick = _.throttle(function() {
      if ($scope.tick) {
        requestAnimationFrame($scope.tick);
      }
      if ($scope.proton) {
        $scope.proton.update();
      }
    }, 33);

    $scope.tick();
  });

  $scope.$on('$ionicView.beforeLeave', function() {
    //sounds.stop($scope.bgm_id);
    sounds.coloring_bgm.stop();
    $scope.signaturePad.off();
    $scope.signaturePad = null;
    $scope.rainbowPad.off();
    $scope.rainbowPad = null;
    $scope.tick = null;
    $scope.renderer.stop();
    $scope.renderer = null;
    $scope.proton.destory();
    $scope.proton = null;
  });

  $scope.platform = PLATFORM;
  $scope.page = $stateParams.page;

  $scope.back = function() {
    $timeout(function() {
      $state.go('player');
    });
  };

  $scope.prev = function() {
    var canvas = document.getElementById('coloring-canvas');
    canvas.getContext('2d').clearRect(0, 0, canvas.width, canvas.height);
    $('#slick').slick('slickPrev');
  };

  $scope.next = function() {
    var canvas = document.getElementById('coloring-canvas');
    canvas.getContext('2d').clearRect(0, 0, canvas.width, canvas.height);
    $('#slick').slick('slickNext');
  };

  $scope.save = function() {
    var canvas = document.getElementById('coloring-canvas'),
        tmp = document.getElementById('tmp-canvas'),
        ctx = canvas.getContext('2d'),
        img = new Image;

    img.onload = function() {
      if ('canvas2ImagePlugin' in window) {

        var tmpCtx = tmp.getContext('2d');

        tmp.width = tmp.width; // clear image
        tmpCtx.fillStyle = '#ffffff';
        tmpCtx.fillRect(0, 0, tmp.width, tmp.height);

        tmpCtx.drawImage(canvas, 0, 0);
        drawImageProp(tmpCtx, this, 0, 0, tmp.width, tmp.height);
        window.canvas2ImagePlugin.saveImageDataToLibrary(function(msg) {
          $ionicPopup.alert({
            title: 'アルバムに保存しました'
          });
        }, function(err) {
          $ionicPopup.alert({
            title: '保存に失敗しました',
            template: 'アルバムの許可設定を確認してください'
          });
        }, tmp);
      }
    };
    img.src = $scope.images[$('#slick').slick('slickCurrentSlide')];
  };

  $scope.images = [
    'img/coloring-1.png',
    'img/coloring-2.png',
    'img/coloring-3.png',
    'img/coloring-free.png'
  ];

  $scope.coloredImages = [
    'img/coloring-1-colored.png',
    'img/coloring-2-colored.png',
    'img/coloring-3-colored.png',
    'img/coloring-free.png'
  ].map(function(src) {
    var img = new Image();
    img.src = src;
    return img;
  });

  $scope.colors = [
    '#FFFFFF',
    '#DB0505','#0071BC','#FBB03B','#FCD5C5','#ECDFB4','#C54D00'
  ];
  $scope.currentColor = $scope.colors[1];
  $scope.pickColor = function(color) {
    $scope.currentColor = color;
    if (color == 'rainbow') {
      // rainbowペン実装
    } else {
      $scope.signaturePad.penColor = color;
    }
  };
})

/**
 * ペアレントゲートコントローラー
 */
.controller('ParentGateCtrl', function($scope, $timeout) {
  $scope.$root.$watch('parentGateExist', function(current, prev) {
    if (!prev && current) {
      reset();
    }
  });

  /**
   * 数式をリセットする
   * @param {boolean} animate リセット時に数式をshakeするかどうか
   */
  function reset(animate) {
    $scope.equationMembers = [
      _.random(6,9), // 6の段〜9の段
      _.random(3,9)  // 掛ける数は3以上
    ];

    if (animate) {
      $scope.shakeEquation = true;
      $timeout(function() {
        $scope.shakeEquation = false;
        $scope.answer = '';
        $scope.equation = [
          $scope.equationMembers[0],
          'x',
          $scope.equationMembers[1],
          '='
        ].join(' ');
      }, 600);
    } else {
      $scope.answer = '';
      $scope.equation = [
        $scope.equationMembers[0],
        'x',
        $scope.equationMembers[1],
        '='
      ].join(' ');
    }
  }

  /**
   * 数式の回答があっているかどうかを返す
   * @return {boolean}
   */
  function checkAnswer() {
    return $scope.answer == $scope.equationMembers.reduce(function(sum, num) {
      return sum * num;
    }).toString();
  }

  /**
   * 数字タップ時
   * @param {number} n タップした数値
   */
  $scope.tapNumber = function(n) {
    $scope.answer += n.toString();
    if ($scope.answer.length == 2) {
      if (checkAnswer()) {
        $scope.closeParentGate(/*success: */ true);
      } else {
        reset(/*animate: */ true);
      }
    }
  };

/**
   * 広告タップ時
   * @param {number} n タップした数値
   */
  $scope.$root.tapAd = function() {
    $scope.$root.openParentGate().then(function() {
      window.open("https://jajajajan-single-app-manager.herokuapp.com/infomations/adurl?platform=" + ionic.Platform.platform(), "_system")
    }).catch(function() {
      if ('cordova' in window && cordova.plugins && cordova.plugins.InterstitialAd && $scope.admobHomeBannerEnabled) {
          cordova.plugins.InterstitialAd.showHomeBanner(function() {
            $timeout(function() {
              $scope.homeBannerIsVisible = true;
            });
          });
      }
    });
  };

  // コントローラー初期化時に数式リセット
  reset();
})

.directive('soundFx', function() {
  return {
    restrict: 'C',
    link: function($scope, $element, $attrs) {
      var sound = null;

      if ($element.hasClass('sound-fx-button')) {
        sound = 'button';
      }
      if ($element.hasClass('sound-fx-close')) {
        sound = 'close'
      }
      if ($element.hasClass('sound-fx-camera')) {
        sound = 'camera';
      }

      $element.bind('touchend', function() {
        // sounds.play(sound);
        sounds[sound].play();
      });
    }
  };
})

.directive('buttonHasState', function($timeout) {
  return {
    restrict: 'C',
    link: function($scope, $element, $attrs) {
      $element.bind('touchstart', function() {
        $element.addClass('active');
      });

      $element.bind('touchend', function() {
        $timeout(function() {
          $element.removeClass('active');
        }, 100);
      });
    }
  };
})

.directive('externalLink', function() {
  return {
    restrict: 'A',
    link: function(scope, element, attrs) {
      return element.bind('click', function(event) {
        if (this.href) {
          event.preventDefault();
          return window.open(this.href, '_system');
        }
      });
    }
  };
})

.run(function($ionicPlatform, $rootScope, $state, $ionicHistory, $http, $q, TRACKING_UA, SKU, PLATFORM) {
  $rootScope.lang = 'ja';
  $rootScope.isLaunch = false;
  $rootScope.adImageUrl = "https://jajajajan-single-app-manager.herokuapp.com/infomations/image?platform=" + PLATFORM;
  $rootScope.splashVideoURL = 'media/opening_ja.mp4';
  $rootScope.jaVideoURL     = 'media/ja.mp4';
  $rootScope.enVideoURL     = 'media/en.mp4';
  $rootScope.menu = {
    terms: '利用規約',
    privacy: 'プライバシーポリシー',
    company: '会社概要',
    contact: 'お問合せ'
  };

  // resumeイベント登録
  document.addEventListener('resume', function() {
    $rootScope.db.upsert('preferences', function(preferences) {
      // 起動回数カウント
      if (!preferences.launchCount) {
        preferences.launchCount = 0;
      }
      preferences.launchCount++;
      return preferences;
    }).then(function() {
      $rootScope.$broadcast('application.resume');
    });
  });

  // pauseイベント登録
  document.addEventListener('pause', function() {
    $rootScope.$broadcast('application.pause');
  });

  // ローカルHTTPサーバー起動
  // RK30board端末でのローカル動画再生不具合回避のため
  function startServer(httpd, port) {
    if (!httpd) {
      $ionicHistory.nextViewOptions({
        disableAnimate: true,
        disableBack: true,
        historyRoot: true
      });
      $state.go('player');
      return;
    }

    port = port || 8967;

    httpd.getURL(function(url) {
      if (url) { return; }

      httpd.startServer({
        'port': port
      }, function() {
        $rootScope.splashVideoURL = 'http://localhost:' + port + '/' + $rootScope.splashVideoURL;
        $rootScope.jaVideoURL     = 'http://localhost:' + port + '/' + $rootScope.jaVideoURL;
        $rootScope.enVideoURL     = 'http://localhost:' + port + '/' + $rootScope.enVideoURL;
        $rootScope.serverStarted = true;
        $ionicHistory.nextViewOptions({
          disableAnimate: true,
          disableBack: true,
          historyRoot: true
        });
        $state.go('player');
      }, function() {
        console.log('server failed to start');
        startServer(httpd, port + 1);
      });
    });
  }

  /**
   * remoteリソースをローカルのpathに保存
   */
  function ensureFile(path, remote) {
    // 実行中のPromiseオブジェクトがあればそれを返す
    if (path in ensureFile) {
      return ensureFile[path];
    }

    var deferred = $q.defer();
    ensureFile[path] = deferred.promise;

    $http.head(path).success(function() {
      deferred.resolve(path);
    }).error(function() {
      (new FileTransfer()).download(remote, path, function(file) {
        deferred.resolve(path);
      }, function() {
        deferred.reject();
        delete ensureFile[path];
      });
    });

    return ensureFile[path];
  }

  function requestVideoFiles() {
    var promises = [];
    var jaLocal = cordova.file.externalDataDirectory + 'ja.mp4';
    var jaRemote = 'https://jajajajan-single-app.s3.amazonaws.com/threepigs-4Dd5y9roXK/ja.mp4';
    $rootScope.jaVideoURL = jaRemote;

    promises.push(ensureFile(jaLocal, jaRemote).then(function() {
      $rootScope.jaVideoURL = jaLocal;
      document.querySelector('#video-ja').src = jaLocal;
    }));

    var enLocal = cordova.file.externalDataDirectory + 'en.mp4';
    var enRemote = 'https://jajajajan-single-app.s3.amazonaws.com/threepigs-4Dd5y9roXK/en.mp4';
    $rootScope.enVideoURL = enRemote;

    promises.push(ensureFile(enLocal, enRemote).then(function() {
      $rootScope.enVideoURL = enLocal;
      document.querySelector('#video-en').src = enLocal;
    }));

    return promises;
  }
  $rootScope.requestVideoFiles = requestVideoFiles;

  $ionicPlatform.ready(function() {
    // Hide the accessory bar by default (remove this to show the accessory bar above the keyboard
    // for form inputs)
    if(window.cordova && window.cordova.plugins.Keyboard) {
      cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
    }
    if(window.StatusBar) {
      StatusBar.styleDefault();
    }

    // Analytics
    if (window.analytics) {
      window.analytics.startTrackerWithId(TRACKING_UA);
      window.analytics.addCustomDimension('1', SKU);
    }

    // .mp3ファイルのsprite
    //
    // NOTE:
    //   .mp3ファイルの結合方法
    //   @see https://github.com/tonistiigi/audiosprite
    window.sounds = {
      button: new buzz.sound('media/button', {formats: ['mp3']}),
      camera: new buzz.sound('media/camera', {formats: ['mp3']}),
      close: new buzz.sound('media/close', {formats: ['mp3']}),
      drawing: new buzz.sound('media/drawing', {formats: ['mp3']}),
      coloring_bgm: new buzz.sound('media/coloring_bgm', {formats: ['mp3']})
    };
    //window.sounds = new Howl({
    //  urls: ['media/sounds.caf', 'media/sounds.mp3'],
    //  sprite: {
    //    "button":       [0,     445],
    //    "camera":       [2000,  1080],
    //    "close":        [5000,  757],
    //    "coloring_bgm": [7000,  35400, true],
    //    "drawing":      [44000, 1800, true]
    //  }
    //});

    $rootScope.db = new PouchDB('localDB', {
      adapter: 'websql', auto_compaction: true
    });
    $rootScope.db.upsert('preferences', function(preferences) {
      // 起動回数カウント
      if (!preferences.launchCount) {
        preferences.launchCount = 0;
      }
      preferences.launchCount++;
      return preferences;
    }).then(function() {
      $rootScope.$broadcast('application.resume');
    });

    // 端末の言語設定取得
    if (navigator.globalization) {
      navigator.globalization.getPreferredLanguage(function(lang) {
        if (lang.value == 'ja' || /^ja[-_]/.test(lang.value)) {
          $rootScope.lang = 'ja';
          $rootScope.splashVideoURL = 'media/opening_ja.mp4';
          $rootScope.menu = {
            terms: '利用規約',
            privacy: 'プライバシーポリシー',
            company: '会社概要',
            contact: 'お問合せ'
          };
        } else {
          $rootScope.lang = 'en';
          $rootScope.splashVideoURL = 'media/opening_en.mp4';
          $rootScope.menu = {
            terms: 'Terms of service',
            privacy: 'Privacy policy',
            company: 'Corporate profile',
            contact: 'Contact us'
          };
        }
        if (window.analytics) {
          window.analytics.addCustomDimension('2', $rootScope.lang);
        }

        // タイミングによってはすでに$digestが走っているので
        // try/catchで対処
        try {
          $rootScope.$digest();
        } catch (e) {
        }
      });
    }

    if (PLATFORM == 'bandai') {
      startServer(('cordova' in window) && cordova.plugins && cordova.plugins.CorHttpd);
    } else {
      if (PLATFORM == 'android') {
        requestVideoFiles();
      }

      $ionicHistory.nextViewOptions({
        disableAnimate: true,
        disableBack: true,
        historyRoot: true
      });
      $state.go('player');
    }

    if ((window.innerWidth / window.innerHeight) > 1.5) {
      document.body.className += ' screen-ratio-wide';
    }

    if (window.innerWidth <= 480) {
      document.body.className += ' screen-resolution-s';
    } else if (window.innerWidth <= 780) {
      document.body.className += ' screen-resolution-m';
    } else {
      document.body.className += ' screen-resolution-l';
      ionic.Platform.isTablet = true;
    }
  });
})
;

/**
 * By Ken Fyrstenberg
 *
 * drawImageProp(context, image [, x, y, width, height [,offsetX, offsetY]])
 *
 * If image and context are only arguments rectangle will equal canvas
*/
function drawImageProp(ctx, img, x, y, w, h, offsetX, offsetY) {
    if (arguments.length === 2) {
        x = y = 0;
        w = ctx.canvas.width;
        h = ctx.canvas.height;
    }

    /// default offset is center
    offsetX = offsetX ? offsetX : 0.5;
    offsetY = offsetY ? offsetY : 0.5;

    /// keep bounds [0.0, 1.0]
    if (offsetX < 0) offsetX = 0;
    if (offsetY < 0) offsetY = 0;
    if (offsetX > 1) offsetX = 1;
    if (offsetY > 1) offsetY = 1;

    var iw = img.width,
        ih = img.height,
        r = Math.min(w / iw, h / ih),
        nw = iw * r,   /// new prop. width
        nh = ih * r,   /// new prop. height
        cx, cy, cw, ch, ar = 1;

    /// decide which gap to fill    
    if (nw < w) ar = w / nw;
    if (nh < h) ar = h / nh;
    nw *= ar;
    nh *= ar;

    /// calc source rectangle
    cw = iw / (nw / w);
    ch = ih / (nh / h);

    cx = (iw - cw) * offsetX;
    cy = (ih - ch) * offsetY;

    /// make sure source rectangle is valid
    if (cx < 0) cx = 0;
    if (cy < 0) cy = 0;
    if (cw > iw) cw = iw;
    if (ch > ih) ch = ih;

    /// fill image in dest. rectangle
    ctx.drawImage(img, cx, cy, cw, ch,  x, y, w, h);
}
